# Scotty TNG - ping/snmp based realtime network monitoring
#
# $Id: view.rb 90 2008-09-26 13:30:56Z liske $
#
# Authors:
#   Thomas Liske <liske@ibh.de>
#   Andre Ellguth <ellguth@ibh.de>
#
# Copyright Holder:
#   2008 (C) IBH IT-Service GmbH [http://www.ibh.de/]
#
# License:
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this package; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
#

class View
    @@views = Array.new

    def initialize(notebook, xview, &progress)
	@status = "ok"
	@ostatus = Hash.new
	@sstatus = {"ok" => 0, "info" => 0, "warning" => 0, "critical" => 0}

	@title = xview.attributes["title"]

	@scrolledwin = Gtk::ScrolledWindow.new
	@layout = Gtk::Layout.new
	@scrolledwin.add(@layout)

	@eb = Gtk::EventBox.new
	@label = Gtk::Label.new(@title)
	@eb.add(@label)
	@eb.show_all

	# Setup layout size
	@width = xview.attributes["width"].to_i
	@height = xview.attributes["height"].to_i

	@layout.set_size(@width, @height)
	@layout_img = Cairo::ImageSurface.new(Cairo::FORMAT_ARGB32, @width, @height)
	@layout_ct = Cairo::Context.new(@layout_img)

	@nodes = Hash.new
	@charts = Hash.new
	@networks = Hash.new

	# Add text elements
	xview.elements.each("text") { |text|
	    label = Gtk::Label.new
	    if text.attributes["color"] != nil then
		label.markup = "<span foreground='"+ text.attributes['color'] +"'>#{text.text}</span>"
	    else
		label.text = text.text
	    end
	    @layout.put(label, text.attributes["x"].to_i, text.attributes["y"].to_i)
	    progress.call
	}

	# Add node elements
	xview.elements.each("node") { |node|
	    @nodes[node.attributes["name"]] = VNode.new(self, node, @layout)
	    progress.call
	}

	# Add chart elements
	xview.elements.each("chart") { |chart|
	    @charts[[chart.attributes["nname"], chart.attributes["sname"]]] = VChart.new(self, chart, @layout)
	    progress.call
	}

	# Draw networks on background
	xview.elements.each("network") { |net|
	    name = net.attributes["name"]
	    x = net.attributes["x"].to_i
	    y = net.attributes["y"].to_i
	    dx = net.attributes["dx"].to_i
	    dy = net.attributes["dy"].to_i

	    unless (x == 0 and y == 0 and dx == 0 and dy == 0) or (name == nil) then
		@networks[name] = [x, y, dx, dy]

		eb = Gtk::EventBox.new
		eb.above_child = true
		eb.visible_window = false
		da = Gtk::DrawingArea.new
		eb.add(da)
		da.set_size_request(dx+2, dy+2)

		img = Cairo::ImageSurface.new(Cairo::FORMAT_ARGB32, dx+2, dy+2)
		ct = Cairo::Context.new(img)

		Client.set_ct_scol(ct, net.attributes["color"].to_s)
		ct.rectangle(0, 0, dx+2, dy+2)
		ct.fill

		da.signal_connect("expose_event") { |widget, arg0|
		    c = widget.window.create_cairo_context
		    c.set_source(img, 0, 0)
		    c.paint

		    false
		}

		begin
		    # Older GTK bindings lack tooltip_markup property
		    eb.tooltip_markup = "<b>#{name}</b>"
		rescue
		    # Gtk::DrawingArea has now window, use the Gtk::EventBox for tooltips
		    Client.getTooltips.set_tip(eb, name, nil)
		end

		@layout.put(eb, x-1, y-1)
	    end
	    progress.call
	}

	# Draw links between nodes on background
	@layout_ct.line_width = 1
	xview.elements.each("link") { |link|
	    Client.set_ct_scol(@layout_ct, link.attributes["color"].to_s)
	    x1 = 0
	    y1 = 0
	    n1 = link.attributes["n1"]

	    if @nodes[n1] then
		x1 = @nodes[n1].x
		y1 = @nodes[n1].y
	    elsif @networks[n1] then
		x1 = @networks[n1][0] + @networks[n1][2]/2
		y1 = @networks[n1][1] + @networks[n1][3]/2
	    end

	    x2 = 0
	    y2 = 0
	    n2 = link.attributes["n2"]

	    if @nodes[n2] then
		x2 = @nodes[n2].x
		y2 = @nodes[n2].y
	    elsif @networks[n2] then
		x2 = @networks[n2][0] + @networks[n2][2]/2
		y2 = @networks[n2][1] + @networks[n2][3]/2
	    end

	    if (x1 != 0 or y1 != 0) and (x2 != 0 or y2 != 0) then
		@layout_ct.move_to(x1, y1)
		@layout_ct.line_to(x2, y2)
		@layout_ct.stroke
	    end

	    progress.call
	}

	@layout.signal_connect("expose_event") do
	    c = @layout.bin_window.create_cairo_context
	    c.set_source(@layout_img, 0, 0)
	    c.paint

	    false
	end

	@layout.signal_connect("button_press_event") do |widget, arg0|
#	    Client.menu.popup(nil, nil, arg0.button, arg0.time) if arg0.button == 3
	end

	notebook.append_page(@scrolledwin, @eb)
	@scrolledwin.show_all

	@@views.push(self)
    end

    def setStatus(oname, status)
	if @ostatus[oname] != status then
	    @sstatus[@ostatus[oname]] = @sstatus[@ostatus[oname]] - 1 if @ostatus[oname]

	    @ostatus[oname] = status

	    @sstatus[status] = @sstatus[status] + 1
	end

	vstatus = "ok"
	sstatus = Array.new
	@sstatus.each_pair{|s, n|
	    vstatus = s if Status.int(s) > Status.int(vstatus) and n > 0

	    if n > 0 then
		sstatus.push("#{s.upcase}(#{n})")
	    end
	}
	@label.text = "#{@title} [#{sstatus.join(', ')}]"

	@eb.modify_bg(Gtk::STATE_NORMAL, Gdk::Color.parse(Status.text_color(vstatus)))
	@eb.modify_bg(Gtk::STATE_ACTIVE, Gdk::Color.parse(Status.text_color(vstatus)))
    end

end
