#!/usr/bin/ruby -w

# Scotty TNG - ping/snmp based realtime network monitoring
#
# $Id: nagios2scotty.rb 89 2008-09-26 12:44:36Z liske $
#
# Authors:
#   Thomas Liske <liske@ibh.de>
#   Andre Ellguth <ellguth@ibh.de>
#
# Copyright Holder:
#   2008 (C) IBH IT-Service GmbH [http://www.ibh.de/apt-dater/]
#
# License:
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this package; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
#
# This little script pushes nagios events into the primary rinda ring
# server tuplespace available. You Scotty TNG and your nagios node names
# should be equal.
#

require 'drb'
require 'rinda/ring'
require 'syslog'

hostname = $*.shift
service = $*.shift
stamp = $*.shift
s = $*.shift
msg = $*.shift

def usage
    puts "Usage: #{$0} $HOSTNAME$ $SERVICEDESC$ $LASTSERVICESTATECHANGE$ $SERVICESTATE$ [$SERVICEOUTPUT$]"
    exit 1
end

usage unless hostname
usage unless service
usage unless s

case s
when "OK", "WARNING", "CRITICAL"
    status = s.downcase
else
    status = "info"
end
stamp = Time.now.to_f unless stamp.to_f > 0


state = [stamp.to_f, status, msg]

Syslog.open('nagios2scotty', Syslog::LOG_PID | Syslog::LOG_NDELAY, Syslog::LOG_LOCAL7)

## Connect to tuple space
ts = nil
DRb.start_service
begin
    ts = Rinda::RingFinger.primary
rescue
    ## Try again
    begin
	ts = Rinda::RingFinger.primary
    rescue
	Syslog.crit("Could not connect to ring server: %s", $!)
	exit 1
    end
end
Syslog.info("Using %s", ts.__drburi)

## Cleanup previous states
begin
    ts.take([:NagiosStatus, hostname, service, nil], 0)
rescue
end

begin
    ts.write([:NagiosStatus, hostname, service, state])
    Syslog.info("Updated %s@%s with status %s.", service, hostname, status)
rescue
    Syslog.crit("Could not push service state change into tuple space: %s", $!)
end
