# Scotty TNG - ping/snmp based realtime network monitoring
#
# $Id: stack.rb 108 2008-10-02 06:38:22Z liske $
#
# Authors:
#   Thomas Liske <liske@ibh.de>
#   Andre Ellguth <ellguth@ibh.de>
#
# Copyright Holder:
#   2008 (C) IBH IT-Service GmbH [http://www.ibh.de/]
#
# License:
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this package; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
#

# This is a stack of +Sensor+ instances of the same class. The sensors might
# be distributed over different nodes.
class Stack
  @@stacks = Hash.new

  attr_reader :name, :sensors

  def initialize(xmlnode)
    @name = xmlnode.attributes["name"]

    puts " " + @name
    print "   "
    @sensors = Hash.new
    @values = 0
    @ctype = nil
    xmlnode.elements.each("sensor") { |xmlsensor|
	node = xmlsensor.attributes["node"]
	name = xmlsensor.attributes["name"]
	sensor = Sensor.getSensor(node, name)

	print "#{node}/#{name} "

	sensor.addStack(self)
	@sensors["#{node}/#{name}"] = 0

	unless @ctype then
	    @ctype = sensor.class.name
	    opts = ScottyTNG.getTS.read [:DRingLOpts, node, name, nil]
	    @opts = opts[3].clone
	    @opts[:title] = @name
	    @opts[:descr] = "<u>#{node}:</u>\n#{@opts[:descr]}"
	else
	    opts = ScottyTNG.getTS.read [:DRingLOpts, node, name, nil]
	    @opts[:descr] += "\n<u>#{node}:</u>\n#{opts[3][:descr]}"
	end
    }
    puts

    @rl = RingList.new(nil, self, @name , @opts)
    
    @@stacks[name] = self
  end

  def self.addStack(xmlstack)
    return Stack.new(xmlstack)
  end

  def self.getStacks()
    return @@stacks
  end
  
  def pushProbe(lname, value)
    @sensors[lname] = value
    @values += 1

    if @values == @sensors.size then
	val = nil
	@sensors.each_pair{|k,v|
	    unless val then
		val = v
	    else
		val.each_index{|i|
		    begin
			case @opts[:format][i]
			when 'b'
			    val[i] = val[i] or v[i]
		        else
			    val[i] = val[i].to_i + v[i].to_i if v[i]
			end
		    rescue
			p $!
		    end
		}
	    end
	}
	@values = 0

	@rl.pushProbe(val)
    end
  end
end
