#!/usr/bin/perl -w

# Scotty TNG - ping/snmp based realtime network monitoring
#
# $Id: tki2view.pl 69 2008-09-21 17:51:23Z liske $
#
# Authors:
#   Thomas Liske <liske@ibh.de>
#   Andre Ellguth <ellguth@ibh.de>
#
# Copyright Holder:
#   2008 (C) IBH IT-Service GmbH [http://www.ibh.de/apt-dater/]
#
# License:
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this package; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
#
# This is a hack for converting *.tki files of the original scotty program. It
# might work, but don't use it seriously.
#

use strict;

my $in = 0;
my @block;

my @nodes;
my @texts;
my @charts;
my %nnames;
my @links;
my %nrefs;
my %irefs;
my %nodes;

while(<>) {
    chomp;

    if ($_ eq '') {
	if ($in == 1) {
	    my $b = join('|', @block);

	    $b =~ /noupdate create (\S+) /;
	    my $type = $1;
	
	    if ($type eq 'NODE') {
		$b =~ /name (\$\S+) (\S+|\{([^|]+)\})\|/;
		my $old = $1;
		my $name = $2;
		$name = $3 if $3;

		$b =~ /address \$\S+ (\S+)\|/;
		my $address = $1;

		$address = &hostname($address) unless($address =~ /^[\d.]+$/);

		$b =~ /move \$\S+ (\S+) (\S+)\|/;
		my $x = int($1);
		my $y = int($2);
		
		$nnames{$old} = $name;
		push(@nodes, "	<node name='$name' x='$x' y='$y'/>");
		$nodes{$address} = $name;
	    }
	    elsif ($type eq 'TEXT') {
		$b =~ /create TEXT \{([^}]+)\} /;
		my $text = $1;
		$b =~ /move \$\S+ (\S+) (\S+)\|/;
		my $x = int($1);
		my $y = int($2);
		$b =~ /color \$\S+ ([a-z]+)/i;
		my $color = $1;
		push(@texts, "	<text x='$x' y='$y' color='$color'>$text</text>");
	    }
	    elsif ($type eq 'LINK') {
		$b =~ /color \$\S+ ([a-z]+)/i;
		my $color = $1;
		$b =~ /create LINK (\S+) (\S+) /;

		my $l = "	<link n1='".$nnames{$1}."' n2='".$nnames{$2}."' color='$color'/>";
		$l = "<!-- $l -->" unless length($nnames{$1}) != 0 and length($nnames{$2}) != 0;
		push(@links, $l);
	    }
	    elsif ($type eq 'STRIPCHART' && $b =~ /round trip time/) {
		$b =~ /name \$\S+ (\S+|\{([^|]+)\})\|/;
		my $node = $1;
		$node = $2 if $2;

		$b =~ /move \$\S+ (\S+) (\S+)\|/;
		my $x = int($1);
		my $y = int($2);

		push(@charts, "	<chart node='$node' sensor='rtt' x='$x' y='$y'/>");
	    }
	    elsif ($type eq 'STRIPCHART' && $b =~ /ifLoad/) {
		$b =~ /name \$(\S+) (\S+|\{([^|]+)\})\|/;
		my $chart = $1;

		$b =~ /move \$\S+ (\S+) (\S+)\|/;
		my $x = int($1);
		my $y = int($2);

		push(@charts, "	<chart node='##n:$chart##' sensor='if[##i:$chart##]' x='$x' y='$y'/>");
	    }
#	    elsif ($type eq 'INTERPRETER' && $b =~ /snmp_monitor\.tcl/) {
#		while($b =~ /(strip[\w\d]+) (\d+)/) {
#		    $refs{$1} = $2;
#		    $b =~ s/$1 $2//;
#		}
#	    }
	    elsif ($type eq 'INTERPRETER' && $b =~ /snmp_monitor\.tcl/) {
		while($b =~ s/ined send [^|]+ restart start_ifload_monitor \d+ job\d+ ([\d.]+) ([^|]+)\|//) {
		    my ($nodeip, $charts) = ($1, $2);
		    while($charts =~ s/\$(strip[\w\d]+) (\d+)//) {
			$nrefs{$1} = $nodeip;
			$irefs{$1} = $2;
		    }
		}
	    }
	}

	$in = 1;
	@block = ();
	next;
    }

    push(@block, $_) if ($in == 1 && $_ ne '');
}

print join("\n", @nodes), "\n\n";
print join("\n", @texts), "\n\n";
print join("\n", @links), "\n\n";

my $charts = join("\n", @charts);
foreach my $s (keys %nrefs) {
    $charts =~ s/##n:$s##/$nodes{$nrefs{$s}}/g;
}
foreach my $s (keys %irefs) {
    $charts =~ s/##i:$s##/$irefs{$s}/g;
}

print "$charts\n\n";
